#!/usr/bin/env node

var fs = require('fs');
var path = require('path');
var fss = require('fs-extra');
var argv = require('minimist')(process.argv.slice(2));
var spawn = require('cross-spawn');
var shell = require("shelljs");
const chalk = require('chalk');

var commands = argv._;
var appName = commands[0];

//Create app
try {
    createApp(appName);
} catch (error) {
    console.error(chalk.red(error));
}

//Create app
function createApp(appName) {

    console.log("Check for package " + chalk.cyan("create-react-app"));

    installNpmPackageGlobal("create-react-app", function (code, command, args) {

        if (errorMessage(code, command, args)) {
            return;
        }

        console.log("\r\nCreate the project structure / install dependencies through " + chalk.cyan("create-react-app ") + chalk.green(appName) + "...");
        console.log(chalk.yellow("Please wait.."));

        createReactApp(appName, function (code, command, args) {
            if (errorMessage(code, command, args)) {
                return;
            }

            shell.cd(appName);
            
            console.log(chalk.cyan("\r\nUpdate project structure and install dependencies for typescript through " + chalk.yellow("react-typescript-scripts")));
            console.log(chalk.yellow("Please wait.."));

            createReactTypescriptApp("react-typescript-scripts", function (code, command, args) {
                if (errorMessage(code, command, args)) {
                    return;
                }

                createTypescriptDependencies();
                console.log(chalk.green("\r\nThe project was successfully prepared."));


                console.log(chalk.cyan("\r\nCreate the first app bundle..."));
                createAppBundle(function (code, command, args) {
                    if (errorMessage(code, command, args)) {
                        return;
                    }

                    console.log(chalk.green("\r\nThe application bundle was successfully created!"));
                    console.log(chalk.cyan("\r\nOpen the application here: " + process.cwd() + "\\index.html"));
                });
            });

        });

    });
}

function createTypescriptDependencies() {
    clearFiles();
    copyFiles();
    modifyPackageJson();
}

function installReactTypescriptDependencies() {}

function modifyPackageJson() {

    var packageJson = process.cwd() + '/package.json';
    var packageJsonFile = require(packageJson);

    var jestConfig = process.cwd() + '/node_modules/react-typescript-scripts/templates/jest-config.json';
    var jestConfigFile = require(jestConfig);

    packageJsonFile.jest = jestConfigFile;

    var scriptsConfig = process.cwd() + '/node_modules/react-typescript-scripts/templates/scripts-config.json';
    var scriptsConfigFile = require(scriptsConfig);

    delete packageJsonFile["scripts"];
    packageJsonFile.scripts = scriptsConfigFile;

    fs.writeFile(packageJson, JSON.stringify(packageJsonFile, null, 4), function (err) {
        if (err) 
            return console.error(err);
        }
    );

}

function copyFiles() {
    var paths = getPathsForCopy();

    paths.forEach(function (file) {
        fss.copySync(process.cwd() + "/node_modules/react-typescript-scripts/" + file.from, file.to);
    });
}

function clearFiles() {
    var paths = getPathsForClear();

    paths.forEach(function (file) {
        fss.removeSync(file.item)
    });
}

function getPathsForCopy() {

    var paths = [

        {
            from: "templates/webpack.config.js",
            to: "webpack.config.js"
        }, {
            from: "templates/tsconfig.json",
            to: "tsconfig.json"
        }, {
            from: "templates/preprocessor.js",
            to: "preprocessor.js"
        }, {
            from: "templates/src/",
            to: "src/"
        }, {
            from: "templates/__mocks__/",
            to: "__mocks__/"
        }, {
            from: "templates/index.html",
            to: "index.html"
        }
    ];

    return paths;
}

function getPathsForClear() {
    var paths = [

        {
            item: "src/"
        }, {
            item: "public/index.html"
        }
    ];

    return paths;
}

function errorMessage(code, command, args) {
    if (code !== 0) {
        console.error('`' + command + ' ' + args.join(' ') + '` failed');
        return true;
    } else {
        return false;
    }
}

function installNpmPackage(packageToInstall, callback) {
    var npmArgs = ['install', packageToInstall, '--save-dev', '--save-exact'].filter(function (e) {
        return e;
    });

    var npmProc = spawn('npm', npmArgs, {stdio: 'inherit'});
    npmProc.on('close', function (code) {
        callback(code, 'npm', npmArgs);
    });
}

function createReactApp(appName, callback) {

    var args = [appName].filter(function (e) {
        return e;
    });

    var createAppProc = spawn('create-react-app', args, {stdio: 'inherit'});

    createAppProc.on('close', function (code) {
        callback(code, 'create-react-app', args);
    });
}

function createReactTypescriptApp(package, callback) {
    installNpmPackage(package, callback);
}

function installNpmPackageGlobal(packageToInstall, callback) {
    var npmArgs = ['install', packageToInstall, '-g'].filter(function (e) {
        return e;
    });

    var npmProc = spawn('npm', npmArgs, {stdio: 'inherit'});
    npmProc.on('close', function (code) {
        callback(code, 'npm', npmArgs);
    });
}

function createAppBundle(callback) {

    var args = ['start'].filter(function (e) {
        return e;
    });

    var createAppProc = spawn('npm', args, {stdio: 'inherit'});

    createAppProc.on('close', function (code) {
        callback(code, 'create-react-app', args);
    });
}